/hg/icedtea7-forest/hotspot: 3 new changesets

enevill at icedtea.classpath.org enevill at icedtea.classpath.org
Tue Dec 15 09:00:30 UTC 2015


changeset 5b6efbae9fea in /hg/icedtea7-forest/hotspot
details: http://icedtea.classpath.org/hg/icedtea7-forest/hotspot?cmd=changeset;node=5b6efbae9fea
author: aph
date: Wed Nov 04 13:38:38 2015 +0100

	8138966: Intermittent SEGV running ParallelGC
	Summary: Add necessary memory fences so that the parallel threads are unable to observe partially filled block tables.
	Reviewed-by: tschatzl


changeset c7679d143590 in /hg/icedtea7-forest/hotspot
details: http://icedtea.classpath.org/hg/icedtea7-forest/hotspot?cmd=changeset;node=c7679d143590
author: enevill
date: Thu Nov 19 15:15:20 2015 +0000

	8143067: aarch64: guarantee failure in javac
	Summary: Fix adrp going out of range during code relocation
	Reviewed-by: aph, kvn


changeset eeb4a3ec4563 in /hg/icedtea7-forest/hotspot
details: http://icedtea.classpath.org/hg/icedtea7-forest/hotspot?cmd=changeset;node=eeb4a3ec4563
author: hshi
date: Tue Nov 24 09:02:26 2015 +0000

	8143285: aarch64: Missing load acquire when checking if ConstantPoolCacheEntry is resolved
	Reviewed-by: roland, aph


diffstat:

 src/cpu/aarch64/vm/assembler_aarch64.cpp                              |  101 +++++----
 src/cpu/aarch64/vm/interp_masm_aarch64.cpp                            |    5 +-
 src/share/vm/gc_implementation/parallelScavenge/psParallelCompact.hpp |    7 +-
 3 files changed, 59 insertions(+), 54 deletions(-)

diffs (208 lines):

diff -r 3f07f893bb63 -r eeb4a3ec4563 src/cpu/aarch64/vm/interp_masm_aarch64.cpp
--- a/src/cpu/aarch64/vm/interp_masm_aarch64.cpp	Fri Nov 20 00:10:48 2015 +0000
+++ b/src/cpu/aarch64/vm/interp_masm_aarch64.cpp	Tue Nov 24 09:02:26 2015 +0000
@@ -197,10 +197,11 @@
   get_cache_and_index_at_bcp(cache, index, bcp_offset, index_size);
   // We use a 32-bit load here since the layout of 64-bit words on
   // little-endian machines allow us that.
-  // n.b. unlike x86 cache alreeady includes the index offset
-  ldrw(bytecode, Address(cache,
+  // n.b. unlike x86 cache already includes the index offset
+  lea(bytecode, Address(cache,
 			 constantPoolCacheOopDesc::base_offset()
 			 + ConstantPoolCacheEntry::indices_offset()));
+  ldarw(bytecode, bytecode);
   const int shift_count = (1 + byte_no) * BitsPerByte;
   ubfx(bytecode, bytecode, shift_count, BitsPerByte);
 }
diff -r 3f07f893bb63 -r eeb4a3ec4563 src/share/vm/gc_implementation/parallelScavenge/psParallelCompact.hpp
--- a/src/share/vm/gc_implementation/parallelScavenge/psParallelCompact.hpp	Fri Nov 20 00:10:48 2015 +0000
+++ b/src/share/vm/gc_implementation/parallelScavenge/psParallelCompact.hpp	Tue Nov 24 09:02:26 2015 +0000
@@ -349,7 +349,7 @@
     HeapWord*            _partial_obj_addr;
     region_sz_t          _partial_obj_size;
     region_sz_t volatile _dc_and_los;
-    bool                 _blocks_filled;
+    bool        volatile _blocks_filled;
 
 #ifdef ASSERT
     size_t               _blocks_filled_count;   // Number of block table fills.
@@ -503,7 +503,9 @@
 inline bool
 ParallelCompactData::RegionData::blocks_filled() const
 {
-  return _blocks_filled;
+  bool result = _blocks_filled;
+  OrderAccess::acquire();
+  return result;
 }
 
 #ifdef ASSERT
@@ -517,6 +519,7 @@
 inline void
 ParallelCompactData::RegionData::set_blocks_filled()
 {
+  OrderAccess::release();
   _blocks_filled = true;
   // Debug builds count the number of times the table was filled.
   DEBUG_ONLY(Atomic::inc_ptr(&_blocks_filled_count));


More information about the distro-pkg-dev mailing list
diff -r 3f07f893bb63 -r eeb4a3ec4563 src/cpu/aarch64/vm/assembler_aarch64.cpp
--- a/src/cpu/aarch64/vm/assembler_aarch64.cpp	Fri Nov 20 00:10:48 2015 +0000
+++ b/src/cpu/aarch64/vm/assembler_aarch64.cpp	Tue Nov 24 09:02:26 2015 +0000
@@ -1597,24 +1597,20 @@
       unsigned offset_lo = dest & 0xfff;
       offset = adr_page - pc_page;
 
-      // We handle 3 types of PC relative addressing
+      // We handle 4 types of PC relative addressing
       //   1 - adrp    Rx, target_page
       //       ldr/str Ry, [Rx, #offset_in_page]
       //   2 - adrp    Rx, target_page
       //       add     Ry, Rx, #offset_in_page
       //   3 - adrp    Rx, target_page (page aligned reloc, offset == 0)
+      //       movk    Rx, #imm16<<32
+      //   4 - adrp    Rx, target_page (page aligned reloc, offset == 0)
       //
       // In the first 2 cases we must check that Rx is the same in the
       // adrp and the subsequent ldr/str or add instruction. Otherwise
       // we could accidentally end up treating a type 3 relocation as
       // a type 1 or 2 just because it happened to be followed by a
       // random unrelated ldr/str or add instruction.
       //
-      // In the case of a type 3 relocation, we know that these are
-      // only generated for the safepoint polling page, the crc table
-      // base or the card type byte map base so we assert as much
-      // and of course that the offset is 0.
-      // 
       // In jdk7 the card type byte map base is aligned on a 1K
       // boundary which may fail to be 4K aligned. In that case the
       // card table load will fall into category 2.
@@ -1628,24 +1624,21 @@
 	Instruction_aarch64::patch(branch + sizeof (unsigned),
 				    21, 10, offset_lo >> size);
 	guarantee(((dest >> size) << size) == dest, "misaligned target");
-        instructions = 2;
+	instructions = 2;
       } else if (Instruction_aarch64::extract(insn2, 31, 22) == 0b1001000100 &&
 		Instruction_aarch64::extract(insn, 4, 0) ==
 			Instruction_aarch64::extract(insn2, 4, 0)) {
 	// add (immediate)
-	assert (((jbyte *)target !=
-		 ((CardTableModRefBS*)(Universe::heap()->barrier_set()))->byte_map_base) ||
-		(offset_lo & 0x3FFl) == 0, "offset must be 0x400 aligned for crc_table");
 	Instruction_aarch64::patch(branch + sizeof (unsigned),
 				   21, 10, offset_lo);
+	instructions = 2;
+      } else if (Instruction_aarch64::extract(insn2, 31, 21) == 0b11110010110 &&
+                   Instruction_aarch64::extract(insn, 4, 0) ==
+                     Instruction_aarch64::extract(insn2, 4, 0)) {
+        // movk #imm16<<32
+        Instruction_aarch64::patch(branch + 4, 20, 5, (uint64_t)target >> 32);
+        offset &= (1<<20)-1;
         instructions = 2;
-      } else {
-	assert((jbyte *)target ==
-		((CardTableModRefBS*)(Universe::heap()->barrier_set()))->byte_map_base ||
-               target == StubRoutines::crc_table_addr() ||
-               (address)target == os::get_polling_page(),
-	       "adrp must be polling page, crc_table or byte map base");
-	assert(offset_lo == 0, "offset must be 0 for polling page, crc_table or byte map base");
       }
     }
     int offset_lo = offset & 3;
@@ -1729,20 +1722,16 @@
       // Return the target address for the following sequences
       //   1 - adrp    Rx, target_page
       //       ldr/str Ry, [Rx, #offset_in_page]
-      //   [ 2 - adrp    Rx, target_page         ] Not handled
-      //   [    add     Ry, Rx, #offset_in_page  ]
+      //   2 - adrp    Rx, target_page
+      //       add     Ry, Rx, #offset_in_page
       //   3 - adrp    Rx, target_page (page aligned reloc, offset == 0)
+      //       movk    Rx, #imm12<<32
+      //   4 - adrp    Rx, target_page (page aligned reloc, offset == 0)
       //
-      // In the case of type 1 we check that the register is the same and
+      // In the first two cases  we check that the register is the same and
       // return the target_page + the offset within the page.
-      //
       // Otherwise we assume it is a page aligned relocation and return
-      // the target page only. The only cases this is generated is for
-      // the safepoint polling page or for the card table byte map base so
-      // we assert as much.
-      //
-      // Note: Strangely, we do not handle 'type 2' relocation (adrp followed
-      // by add) which is handled in pd_patch_instruction above.
+      // the target page only.
       //
       unsigned insn2 = ((unsigned*)insn_addr)[1];
       if (Instruction_aarch64::extract(insn2, 29, 24) == 0b111001 &&
@@ -1752,11 +1741,19 @@
 	unsigned int byte_offset = Instruction_aarch64::extract(insn2, 21, 10);
 	unsigned int size = Instruction_aarch64::extract(insn2, 31, 30);
 	return address(target_page + (byte_offset << size));
+      } else if (Instruction_aarch64::extract(insn2, 31, 22) == 0b1001000100 &&
+                Instruction_aarch64::extract(insn, 4, 0) ==
+                        Instruction_aarch64::extract(insn2, 4, 0)) {
+        // add (immediate)
+        unsigned int byte_offset = Instruction_aarch64::extract(insn2, 21, 10);
+        return address(target_page + byte_offset);
       } else {
-	assert((jbyte *)target_page ==
-		((CardTableModRefBS*)(Universe::heap()->barrier_set()))->byte_map_base ||
-               (address)target_page == os::get_polling_page(),
-	       "adrp must be polling page or byte map base");
+        if (Instruction_aarch64::extract(insn2, 31, 21) == 0b11110010110  &&
+               Instruction_aarch64::extract(insn, 4, 0) ==
+                 Instruction_aarch64::extract(insn2, 4, 0)) {
+          target_page = (target_page & 0xffffffff) |
+                         ((uint64_t)Instruction_aarch64::extract(insn2, 20, 5) << 32);
+        }
 	return (address)target_page;
       }
     } else {
@@ -4776,22 +4773,26 @@
 
 void MacroAssembler::adrp(Register reg1, const Address &dest, unsigned long &byte_offset) {
   relocInfo::relocType rtype = dest.rspec().reloc()->type();
-  if (uabs(pc() - dest.target()) >= (1LL << 32)) {
-    guarantee(rtype == relocInfo::none
-	      || rtype == relocInfo::external_word_type
-	      || rtype == relocInfo::poll_type
-	      || rtype == relocInfo::poll_return_type,
-	      "can only use a fixed address with an ADRP");
-    // Out of range.  This doesn't happen very often, but we have to
-    // handle it
-    mov(reg1, dest);
-    byte_offset = 0;
+  unsigned long low_page = (unsigned long)CodeCache::low_bound() >> 12;
+  unsigned long high_page = (unsigned long)(CodeCache::high_bound()-1) >> 12;
+  unsigned long dest_page = (unsigned long)dest.target() >> 12;
+  long offset_low = dest_page - low_page;
+  long offset_high = dest_page - high_page;
+
+  InstructionMark im(this);
+  code_section()->relocate(inst_mark(), dest.rspec());
+  // 8143067: Ensure that the adrp can reach the dest from anywhere within
+  // the code cache so that if it is relocated we know it will still reach
+  if (offset_high >= -(1<<20) && offset_low < (1<<20)) {
+    _adrp(reg1, dest.target());
   } else {
-    InstructionMark im(this);
-    code_section()->relocate(inst_mark(), dest.rspec());
-    byte_offset = (uint64_t)dest.target() & 0xfff;
-    _adrp(reg1, dest.target());
+    unsigned long pc_page = (unsigned long)pc() >> 12;
+    long offset = dest_page - pc_page;
+    offset = (offset & ((1<<20)-1)) << 12;
+    _adrp(reg1, pc()+offset);
+    movk(reg1, ((unsigned long)dest.target() >> 32) & 0xffff, 32);
   }
+  byte_offset = (unsigned long)dest.target() & 0xfff;
 }
 
 void MacroAssembler::build_frame(int framesize) {
@@ -1597,24 +1597,20 @@
       //
-      // In the first 2 cases we must check that Rx is the same in the
-      // adrp and the subsequent ldr/str or add instruction. Otherwise
-      // we could accidentally end up treating a type 3 relocation as
-      // a type 1 or 2 just because it happened to be followed by a
-      // random unrelated ldr/str or add instruction.
+      // In the first 3 cases we must check that Rx is the same in the adrp and the
+      // subsequent ldr/str, add or movk instruction. Otherwise we could accidentally end
+      // up treating a type 4 relocation as a type 1, 2 or 3 just because it happened
+      // to be followed by a random unrelated ldr/str, add or movk instruction.
       //